# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType

class Seo(models.Model):
    class Meta:
        verbose_name = _('SEO fields')
        verbose_name_plural = _('SEO fields')
        unique_together = (("content_type", "object_id"),)

    title = models.CharField(verbose_name=_('Title'),
        max_length=200, default='', blank=True)
    description = models.CharField(verbose_name=_('Description'),
        max_length=200, default='', blank=True)
    keywords = models.CharField(verbose_name=_('Keywords'),
        max_length=1000, default='', blank=True)

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    def __unicode__(self):
        return self.title

class Url(models.Model):
    class Meta:
        verbose_name = _('URL')
        verbose_name_plural = _('URLs')

    url = models.CharField(verbose_name=_('URL'),
        max_length=200, default='/', unique=True,
        help_text=_("This should be an absolute path, excluding the domain name. Example: '/events/search/'."))

    def get_absolute_url(self):
        return self.url

    def __unicode__(self):
        return self.url



from django import forms
from seo.models import Seo

class SeoForm(forms.ModelForm):
    class Meta:
        model = Seo

    title = forms.CharField(
        required=not Seo._meta.get_field('title').blank,
        widget=forms.Textarea(attrs={'cols': '120', 'rows': '2'}),
        label=Seo._meta.get_field('title').verbose_name,
        initial=Seo._meta.get_field('title').default,
        help_text=Seo._meta.get_field('title').help_text,
        max_length=Seo._meta.get_field('title').max_length,
    )
    description = forms.CharField(
        required=not Seo._meta.get_field('description').blank,
        widget=forms.Textarea(attrs={'cols': '120', 'rows': '2'}),
        label=Seo._meta.get_field('description').verbose_name,
        initial=Seo._meta.get_field('description').default,
        help_text=Seo._meta.get_field('description').help_text,
        max_length=Seo._meta.get_field('description').max_length,
    )
    keywords = forms.CharField(
        required=not Seo._meta.get_field('keywords').blank,
        widget=forms.Textarea(attrs={'cols': '120', 'rows': '5'}),
        label=Seo._meta.get_field('keywords').verbose_name,
        initial=Seo._meta.get_field('keywords').default,
        help_text=Seo._meta.get_field('keywords').help_text,
        max_length=Seo._meta.get_field('keywords').max_length,
    )



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Seo'
        db.create_table('seo_seo', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(default='', max_length=200, blank=True)),
            ('description', self.gf('django.db.models.fields.CharField')(default='', max_length=200, blank=True)),
            ('keywords', self.gf('django.db.models.fields.CharField')(default='', max_length=1000, blank=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal('seo', ['Seo'])

        # Adding model 'Url'
        db.create_table('seo_url', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('url', self.gf('django.db.models.fields.CharField')(default='/', unique=True, max_length=200)),
        ))
        db.send_create_signal('seo', ['Url'])


    def backwards(self, orm):
        
        # Deleting model 'Seo'
        db.delete_table('seo_seo')

        # Deleting model 'Url'
        db.delete_table('seo_url')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'seo.seo': {
            'Meta': {'object_name': 'Seo'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'description': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '1000', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'title': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'seo.url': {
            'Meta': {'object_name': 'Url'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'default': "'/'", 'unique': 'True', 'max_length': '200'})
        }
    }

    complete_apps = ['seo']



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib import admin
from django.contrib.contenttypes import generic
from django.core.exceptions import ImproperlyConfigured
from seo.importpath import importpath
from seo.forms import SeoForm
from seo.models import Seo, Url

class SeoInlines(generic.GenericStackedInline):
    model = Seo
    form = SeoForm
    extra = 1
    max_num = 1

class SeoAdmin(admin.ModelAdmin):
    model = Seo

try:
    admin.site.register(Seo, SeoAdmin)
except admin.sites.AlreadyRegistered:
    pass

class UrlAdmin(admin.ModelAdmin):
    model = Url
    inlines = [SeoInlines]

try:
    admin.site.register(Url, UrlAdmin)
except admin.sites.AlreadyRegistered:
    pass

if not hasattr(settings, 'SEO_FOR_MODELS'):
    raise ImproperlyConfigured('Please add ``SEO_FOR_MODELS = ["<app>.admin.<ModelAdmin>",]`` to your settings.py')

for model_name in settings.SEO_FOR_MODELS:
    model = importpath(model_name, 'SEO_FOR_MODELS')
    try:
        model_admin = admin.site._registry[model].__class__
    except KeyError:
        raise ImproperlyConfigured('Please set ``seo`` in your settings.py only as last INSTALLED_APPS')
    admin.site.unregister(model)

    setattr(model_admin, 'inlines', getattr(model_admin, 'inlines', []))
    if not SeoInlines in model_admin.inlines:
        model_admin.inlines = list(model_admin.inlines)[:] + [SeoInlines]

    admin.site.register(model, model_admin)



# -*- coding: utf-8 -*-
from django import template
from django.contrib.contenttypes.models import ContentType
from django.db.models import Model
from django.utils.html import escape
from seo.models import Seo, Url
import warnings

INTENTS = ['title', 'keywords', 'description', ]

register = template.Library()

class SeoNode(template.Node):
    def __init__(self, intent, object, variable):
        self.intent = intent
        self.object = object
        self.variable = variable

    def _process_var_argument(self, context, seo):
        if self.variable is None:
            return escape(getattr(seo, self.intent, u''))
        else:
            context[self.variable] = getattr(seo, self.intent, None)
            return u''

    def _seo_by_url(self, context):
        try:
            request = context['request']
        except KeyError:
            warnings.warn('`request` was not found in context. Add "django.core.context_processors.request" to `TEMPLATE_CONTEXT_PROCESSORS` in your settings.py.')
            return self._process_var_argument(context, None)
        else:
            try:
                object = Url.objects.get(url=request.path_info)
            except Url.DoesNotExist:
                return self._process_var_argument(context, None)
            else:
                try:
                    seo = Seo.objects.get(
                        content_type=ContentType.objects.get_for_model(
                                object.__class__),
                        object_id=object.id)
                except Seo.DoesNotExist:
                    return self._process_var_argument(context, None)
                else:
                    return self._process_var_argument(context, seo)

    def _seo_by_content_object(self, context):
        object = template.Variable(self.object).resolve(context)
        if not isinstance(object, Model):
            return self._process_var_argument(context, None)

        try:
            seo = Seo.objects.get(
                content_type=ContentType.objects.get_for_model(
                        object.__class__),
                object_id=object.id)
        except Seo.DoesNotExist:
            return self._seo_by_url(context)
        else:
            return self._process_var_argument(context, seo)

    def render(self, context):
        if self.object is None:
            # search by URL
            return self._seo_by_url(context)
        else:
            # Try to retrieve object from context
            return self._seo_by_content_object(context)


def seo_tag(parser, token):
    """Get seo data for object"""
    splited = token.split_contents()
    if splited[1] in INTENTS:
        if len(splited) in [4, 6] and splited[2] == 'for':
            if len(splited) == 4:
                return SeoNode(splited[1], splited[3], None)
            elif splited[4] == 'as':
                return SeoNode(splited[1], splited[3], splited[5])
        elif len(splited) in [2, 4]:
            if len(splited) == 2:
                return SeoNode(splited[1], None, None)
            elif splited[2] == 'as':
                return SeoNode(splited[1], None, splited[3])
    raise template.TemplateSyntaxError, "Invalid syntax. Use ``{% seo <title|keywords|description> [for <object>] [as <variable>] %}``"
register.tag('seo', seo_tag)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding unique constraint on 'Seo', fields ['object_id', 'content_type']
        db.create_unique('seo_seo', ['object_id', 'content_type_id'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'Seo', fields ['object_id', 'content_type']
        db.delete_unique('seo_seo', ['object_id', 'content_type_id'])


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'seo.seo': {
            'Meta': {'unique_together': "(('content_type', 'object_id'),)", 'object_name': 'Seo'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'description': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '1000', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'title': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'seo.url': {
            'Meta': {'object_name': 'Url'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'default': "'/'", 'unique': 'True', 'max_length': '200'})
        }
    }

    complete_apps = ['seo']
